@@ -20,7 +20,7 @@ limitations under the License.
 ]]--
 
 local rspamd_logger = require "rspamd_logger"
-local ts = require("tableshape").types
+local T = require "lua_shape.core"
 local lua_util = require "lua_util"
 
 local exports = {}
@@ -73,7 +73,7 @@ local function maybe_adjust_type(data, mtype)
   }
 
   if mtype == 'callback' then
-    return mtype
+    return data, mtype
   end
 
   for _, t in ipairs(known_types) do
@@ -86,30 +86,73 @@ local function maybe_adjust_type(data, mtype)
   return data, mtype
 end
 
-local external_map_schema = ts.shape {
-  external = ts.equivalent(true), -- must be true
-  backend = ts.string, -- where to get data, required
-  method = ts.one_of { "body", "header", "query" }, -- how to pass input
-  encode = ts.one_of { "json", "messagepack" }:is_optional(), -- how to encode input (if relevant)
-  timeout = (ts.number + ts.string / lua_util.parse_time_interval):is_optional(),
-}
+local external_map_schema = T.table({
+  external = T.literal(true), -- must be true
+  backend = T.string():optional(), -- where to get data, required for HTTP
+  cdb = T.string():optional(), -- path to CDB file, required for CDB
+  method = T.enum({ "body", "header", "query" }):optional(), -- how to pass input
+  encode = T.enum({ "json", "messagepack" }):optional(), -- how to encode input (if relevant)
+  timeout = T.one_of({
+    T.number({ min = 0 }),
+    T.transform(T.string(), lua_util.parse_time_interval)
+  }):optional(),
+})
+
+-- Storage for CDB instances
+local cdb_maps = {}
+local cdb_finisher_set = false
 
 local rspamd_http = require "rspamd_http"
 local ucl = require "ucl"
 
-local function url_encode_string(str)
-  str = string.gsub(str, "([^%w _%%%-%.~])",
-      function(c)
-        return string.format("%%%02X", string.byte(c))
-      end)
-  str = string.gsub(str, " ", "+")
-  return str
+-- Function to handle CDB maps
+local function handle_cdb_map(map_config, key, callback, task)
+  local rspamd_cdb = require "rspamd_cdb"
+  local hash_key = map_config.cdb
+
+  -- Check if we need to open the CDB file
+  if not cdb_maps[hash_key] then
+    local cdb_file = map_config.cdb
+    -- Provide ev_base to monitor changes
+    local cdb_handle = rspamd_cdb.open(cdb_file, task:get_ev_base())
+
+    if not cdb_handle then
+      local err_msg = string.format("Failed to open CDB file: %s", cdb_file)
+      rspamd_logger.errx(task, err_msg)
+      if callback then
+        callback(false, err_msg, 500, task)
+      end
+      return nil
+    else
+      cdb_maps[hash_key] = cdb_handle
+    end
+  end
+
+  -- Look up the key in CDB
+  local result = cdb_maps[hash_key]:find(key)
+
+  if callback then
+    if result then
+      callback(true, result, 200, task)
+    else
+      callback(false, 'not found', 404, task)
+    end
+    return nil
+  end
+
+  return result
 end
 
-assert(url_encode_string('上海+中國') == '%E4%B8%8A%E6%B5%B7%2B%E4%B8%AD%E5%9C%8B')
-assert(url_encode_string('? and the Mysterians') == '%3F+and+the+Mysterians')
-
-local function query_external_map(map_config, upstreams, key, callback, task)
+-- Query external map using HTTP or CDB
+-- task_or_ctx can be either a task object or a context table with:
+--   { config, ev_base, session, resolver } for rspamadm usage
+-- If callback is nil and task_or_ctx is a context table (rspamadm), performs synchronous request
+local function query_external_map(map_config, upstreams, key, callback, task_or_ctx)
+  -- Check if this is a CDB map
+  if map_config.cdb then
+    return handle_cdb_map(map_config, key, callback, task_or_ctx)
+  end
+  -- Fallback to HTTP
   local http_method = (map_config.method == 'body' or map_config.method == 'form') and 'POST' or 'GET'
   local upstream = upstreams:get_upstream_round_robin()
   local http_headers = {
@@ -118,6 +161,12 @@ local function query_external_map(map_config, upstreams, key, callback, task)
   local http_body = nil
   local url = map_config.backend
 
+  -- Determine logging target (task or config)
+  local log_obj = task_or_ctx
+  if type(task_or_ctx) == 'table' and task_or_ctx.config then
+    log_obj = task_or_ctx.config
+  end
+
   if type(key) == 'string' or type(key) == 'userdata' then
     if map_config.method == 'body' then
       http_body = key
@@ -127,7 +176,7 @@ local function query_external_map(map_config, upstreams, key, callback, task)
         key = key
       }
     elseif map_config.method == 'query' then
-      url = string.format('%s?key=%s', url, url_encode_string(tostring(key)))
+      url = string.format('%s?key=%s', url, lua_util.url_encode_string(tostring(key)))
     end
   elseif type(key) == 'table' then
     if map_config.method == 'body' then
@@ -139,10 +188,13 @@ local function query_external_map(map_config, upstreams, key, callback, task)
         http_headers['Content-Type'] = 'application/msgpack'
       else
         local caller = debug.getinfo(2) or {}
-        rspamd_logger.errx(task,
+        rspamd_logger.errx(log_obj,
             "requested external map key with a wrong combination body method and missing encode; caller: %s:%s",
             caller.short_src, caller.currentline)
-        callback(false, 'invalid map usage', 500, task)
+        if callback then
+          callback(false, 'invalid map usage', 500, task_or_ctx)
+        end
+        return nil
       end
     else
       -- query/header and no encode
@@ -150,7 +202,8 @@ local function query_external_map(map_config, upstreams, key, callback, task)
         local params_table = {}
         for k, v in pairs(key) do
           if type(v) == 'string' then
-            table.insert(params_table, string.format('%s=%s', url_encode_string(k), url_encode_string(v)))
+            table.insert(params_table,
+                string.format('%s=%s', lua_util.url_encode_string(k), lua_util.url_encode_string(v)))
           end
         end
         url = string.format('%s?%s', url, table.concat(params_table, '&'))
@@ -158,29 +211,20 @@ local function query_external_map(map_config, upstreams, key, callback, task)
         http_headers = key
       else
         local caller = debug.getinfo(2) or {}
-        rspamd_logger.errx(task,
+        rspamd_logger.errx(log_obj,
             "requested external map key with a wrong combination of encode and input; caller: %s:%s",
             caller.short_src, caller.currentline)
-        callback(false, 'invalid map usage', 500, task)
-        return
+        if callback then
+          callback(false, 'invalid map usage', 500, task_or_ctx)
+        end
+        return nil
       end
     end
   end
 
-  local function map_callback(err, code, body, _)
-    if err then
-      callback(false, err, code, task)
-    elseif code == 200 then
-      callback(true, body, 200, task)
-    else
-      callback(false, err, code, task)
-    end
-  end
-
-  local ret = rspamd_http.request {
-    task = task,
+  -- Build HTTP request options - support both task and rspamadm context
+  local http_opts = {
     url = url,
-    callback = map_callback,
     timeout = map_config.timeout or 1.0,
     keepalive = true,
     upstream = upstream,
@@ -189,8 +233,49 @@ local function query_external_map(map_config, upstreams, key, callback, task)
     body = http_body,
   }
 
+  -- Check if task_or_ctx is a context table (rspamadm) or a task (userdata)
+  -- rspamadm context is a Lua table with ev_base, config, session, resolver fields
+  -- task is userdata (C object), so type(task) ~= 'table'
+  local is_rspamadm_ctx = type(task_or_ctx) == 'table' and task_or_ctx.ev_base and task_or_ctx.config
+  if is_rspamadm_ctx then
+    -- rspamadm context
+    http_opts.config = task_or_ctx.config
+    http_opts.ev_base = task_or_ctx.ev_base
+    http_opts.session = task_or_ctx.session
+    http_opts.resolver = task_or_ctx.resolver
+  elseif task_or_ctx then
+    -- Regular task (userdata)
+    http_opts.task = task_or_ctx
+  end
+
+  -- If no callback and rspamadm context, use coroutine-based synchronous mode
+  if not callback and is_rspamadm_ctx then
+    local err, response = rspamd_http.request(http_opts)
+    if err then
+      return nil
+    elseif response and response.code == 200 then
+      return response.content
+    else
+      return nil
+    end
+  end
+
+  -- Async mode with callback
+  local function map_callback(err, code, body, _)
+    if err then
+      callback(false, err, code, task_or_ctx)
+    elseif code == 200 then
+      callback(true, body, 200, task_or_ctx)
+    else
+      callback(false, err, code, task_or_ctx)
+    end
+  end
+
+  http_opts.callback = map_callback
+  local ret = rspamd_http.request(http_opts)
+
   if not ret then
-    callback(false, 'http request error', 500, task)
+    callback(false, 'http request error', 500, task_or_ctx)
   end
 end
 
@@ -206,24 +291,33 @@ end
 --]]
 local function rspamd_map_add_from_ucl(opt, mtype, description, callback)
   local ret = {
-    get_key = function(t, k, key_callback, task)
+    -- get_key supports both task (userdata) and rspamadm context (table with ev_base, config, session, resolver)
+    -- For external maps with rspamadm context (no callback), uses coroutine-based synchronous request
+    get_key = function(t, k, key_callback, task_or_ctx)
       if t.__data then
         local cb = key_callback or callback
         if t.__external then
-          if not cb or not task then
+          -- Check if this is rspamadm context with no callback - use sync mode
+          -- rspamadm context is a Lua table; task is userdata (C object)
+          local is_rspamadm_ctx = type(task_or_ctx) == 'table' and task_or_ctx.ev_base and task_or_ctx.config
+          if not cb and is_rspamadm_ctx then
+            -- Coroutine-based synchronous external map query for rspamadm
+            return query_external_map(t.__data, t.__upstreams, k, nil, task_or_ctx)
+          elseif not cb or not task_or_ctx then
             local caller = debug.getinfo(2) or {}
-            rspamd_logger.errx(rspamd_config, "requested external map key without callback or task; caller: %s:%s",
+            rspamd_logger.errx(rspamd_config, "requested external map key without callback or task/context; caller: %s:%s",
                 caller.short_src, caller.currentline)
             return nil
+          else
+            query_external_map(t.__data, t.__upstreams, k, cb, task_or_ctx)
           end
-          query_external_map(t.__data, t.__upstreams, k, cb, task)
         else
           local result = t.__data:get_key(k)
           if cb then
             if result then
-              cb(true, result, 200, task)
+              cb(true, result, 200, task_or_ctx)
             else
-              cb(false, 'not found', 404, task)
+              cb(false, 'not found', 404, task_or_ctx)
             end
           else
             return result
@@ -241,9 +335,9 @@ local function rspamd_map_add_from_ucl(opt, mtype, description, callback)
     end
   }
   local ret_mt = {
-    __index = function(t, k, key_callback, task)
+    __index = function(t, k, key_callback, task_or_ctx)
       if t.__data then
-        return t.get_key(k, key_callback, task)
+        return t.get_key(k, key_callback, task_or_ctx)
       end
 
       return nil
@@ -263,6 +357,10 @@ local function rspamd_map_add_from_ucl(opt, mtype, description, callback)
 
   if type(opt) == 'string' then
     opt, mtype = maybe_adjust_type(opt, mtype)
+    if not opt or opt == '' then
+      rspamd_logger.errx(rspamd_config, 'invalid map url (nil or empty): %s', description)
+      return nil
+    end
     local cache_key = map_hash_key(opt, mtype)
     if not callback and maps_cache[cache_key] then
       rspamd_logger.infox(rspamd_config, 'reuse url for %s(%s)',
@@ -317,7 +415,7 @@ local function rspamd_map_add_from_ucl(opt, mtype, description, callback)
 
         if string.find(opt[1], '^%d') then
           -- List of numeric stuff (hope it's ipnets definitions)
-          local map = rspamd_config:radix_from_ucl(opt)
+          local map = rspamd_config:radix_from_ucl(opt, description)
 
           if map then
             ret.__data = map
@@ -457,20 +555,42 @@ local function rspamd_map_add_from_ucl(opt, mtype, description, callback)
     else
       if opt.external then
         -- External map definition, missing fields are handled by schema
-        local parse_res, parse_err = external_map_schema(opt)
+        local parse_res, parse_err = external_map_schema:transform(opt)
 
         if parse_res then
-          ret.__upstreams = lua_util.http_upstreams_by_url(rspamd_config:get_mempool(), opt.backend)
-          if ret.__upstreams then
+          if opt.cdb then
             ret.__data = opt
             ret.__external = true
             setmetatable(ret, ret_mt)
             maybe_register_selector()
 
+            if not cdb_finisher_set then
+              -- Register a finalize script to close all CDB handles when Rspamd stops
+              rspamd_config:register_finish_script(function()
+                for path, _ in pairs(cdb_maps) do
+                  rspamd_logger.infox(rspamd_config, 'closing CDB map: %s', path)
+                  cdb_maps[path] = nil
+                end
+              end)
+              cdb_finisher_set = true
+            end
+
             return ret
+          elseif opt.backend then
+            ret.__upstreams = lua_util.http_upstreams_by_url(rspamd_config:get_mempool(), opt.backend)
+            if ret.__upstreams then
+              ret.__data = opt
+              ret.__external = true
+              setmetatable(ret, ret_mt)
+              maybe_register_selector()
+
+              return ret
+            else
+              rspamd_logger.errx(rspamd_config, 'cannot parse external map upstreams: %s',
+                  opt.backend)
+            end
           else
-            rspamd_logger.errx(rspamd_config, 'cannot parse external map upstreams: %s',
-                opt.backend)
+            rspamd_logger.errx(rspamd_config, 'external map requires either "cdb" or "backend" parameter')
           end
         else
           rspamd_logger.errx(rspamd_config, 'cannot parse external map: %s',
@@ -538,15 +658,12 @@ local function rspamd_maybe_check_map(key, what)
       return rspamd_maybe_check_map(key, elt)
     end, what)
   end
-  if type(rspamd_maps) == "table" then
-    local mn
-    if starts(key, "map:") then
-      mn = string.sub(key, 5)
-    elseif starts(key, "map://") then
-      mn = string.sub(key, 7)
+  if type(rspamd_maps) == "table" and starts(key, "map:") then
+    local mn = string.sub(key, 5)
+    if starts(mn, "//") then
+      mn = string.sub(mn, 3)
     end
-
-    if mn and rspamd_maps[mn] then
+    if rspamd_maps[mn] then
       return rspamd_maps[mn]:get_key(what)
     end
   end
@@ -593,27 +710,28 @@ exports.fill_config_maps = function(mname, opts, map_defs)
   return true
 end
 
-local direct_map_schema = ts.shape { -- complex object
-  name = ts.string:is_optional(),
-  description = ts.string:is_optional(),
-  selector_alias = ts.string:is_optional(), -- an optional alias for the selectos framework
-  timeout = ts.number,
-  data = ts.array_of(ts.string):is_optional(),
-  -- Tableshape has no options support for something like key1 or key2?
-  upstreams = ts.one_of {
-    ts.string,
-    ts.array_of(ts.string),
-  }             :is_optional(),
-  url = ts.one_of {
-    ts.string,
-    ts.array_of(ts.string),
-  }       :is_optional(),
-}
+local direct_map_schema = T.table({ -- complex object
+  name = T.string():optional(),
+  description = T.string():optional(),
+  selector_alias = T.string():optional(), -- an optional alias for the selectors framework
+  timeout = T.number(),
+  data = T.array(T.string()):optional(),
+  -- Can specify either upstreams or url
+  upstreams = T.one_of({
+    T.string(),
+    T.array(T.string()),
+  }):optional(),
+  url = T.one_of({
+    T.string(),
+    T.array(T.string()),
+  }):optional(),
+})
 
-exports.map_schema = ts.one_of {
-  ts.string, -- 'http://some_map'
-  ts.array_of(ts.string), -- ['foo', 'bar']
-  ts.one_of { direct_map_schema, external_map_schema }
-}
+exports.map_schema = T.one_of({
+  T.string(), -- 'http://some_map'
+  T.array(T.string()), -- ['foo', 'bar']
+  direct_map_schema, -- complex object with url/upstreams
+  external_map_schema -- external backend (HTTP API, etc)
+})
 
 return exports
