@@ -21,6 +21,7 @@ local fun = require "fun"
 local rspamd_util = require "rspamd_util"
 local rspamd_parsers = require "rspamd_parsers"
 local rspamd_regexp = require "rspamd_regexp"
+local rspamd_logger = require "rspamd_logger"
 local lua_util = require "lua_util"
 local bit = require "bit"
 local rspamd_url = require "rspamd_url"
@@ -124,43 +125,50 @@ rspamd_config:register_symbol({
   parent = date_id,
 })
 
-local obscured_id = rspamd_config:register_symbol {
-  callback = function(task)
-    local susp_urls = task:get_urls_filtered({ 'obscured', 'zw_spaces' })
+-- Check if url_suspect plugin is enabled (it provides better R_SUSPICIOUS_URL)
+local url_suspect_enabled = rspamd_config:get_all_opt('url_suspect')
+if url_suspect_enabled and url_suspect_enabled.enabled ~= false then
+  rspamd_logger.infox(rspamd_config, 'url_suspect plugin is enabled, skipping old R_SUSPICIOUS_URL registration')
+else
+  -- Legacy R_SUSPICIOUS_URL implementation (kept for backward compatibility)
+  local obscured_id = rspamd_config:register_symbol {
+    callback = function(task)
+      local susp_urls = task:get_urls_filtered({ 'obscured', 'zw_spaces' })
 
-    if susp_urls and susp_urls[1] then
-      local obs_flag = url_flags_tab.obscured
-      local zw_flag = url_flags_tab.zw_spaces
+      if susp_urls and susp_urls[1] then
+        local obs_flag = url_flags_tab.obscured
+        local zw_flag = url_flags_tab.zw_spaces
 
-      for _, u in ipairs(susp_urls) do
-        local fl = u:get_flags_num()
-        if bit.band(fl, obs_flag) ~= 0 then
-          task:insert_result('R_SUSPICIOUS_URL', 1.0, u:get_host())
-        end
-        if bit.band(fl, zw_flag) ~= 0 then
-          task:insert_result('ZERO_WIDTH_SPACE_URL', 1.0, u:get_host())
+        for _, u in ipairs(susp_urls) do
+          local fl = u:get_flags_num()
+          if bit.band(fl, obs_flag) ~= 0 then
+            task:insert_result('R_SUSPICIOUS_URL', 1.0, u:get_host())
+          end
+          if bit.band(fl, zw_flag) ~= 0 then
+            task:insert_result('ZERO_WIDTH_SPACE_URL', 1.0, u:get_host())
+          end
         end
       end
-    end
 
-    return false
-  end,
-  name = 'R_SUSPICIOUS_URL',
-  score = 5.0,
-  one_shot = true,
-  description = 'A message has been identified to contain an obfuscated or suspicious URL',
-  group = 'url'
-}
+      return false
+    end,
+    name = 'R_SUSPICIOUS_URL',
+    score = 5.0,
+    one_shot = true,
+    description = 'A message has been identified to contain an obfuscated or suspicious URL',
+    group = 'url'
+  }
 
-rspamd_config:register_symbol {
-  type = 'virtual',
-  name = 'ZERO_WIDTH_SPACE_URL',
-  score = 7.0,
-  one_shot = true,
-  description = 'Zero width space in URL',
-  group = 'url',
-  parent = obscured_id,
-}
+  rspamd_config:register_symbol {
+    type = 'virtual',
+    name = 'ZERO_WIDTH_SPACE_URL',
+    score = 7.0,
+    one_shot = true,
+    description = 'Zero width space in URL',
+    group = 'url',
+    parent = obscured_id,
+  }
+end
 
 rspamd_config.ENVFROM_PRVS = {
   callback = function(task)
@@ -807,7 +815,6 @@ local rnds_check_id = rspamd_config:register_symbol {
       -- Try to resolve
       local task_ip = task:get_ip()
       if task_ip and task_ip:is_valid() then
-        local rspamd_logger = require "rspamd_logger"
         local function rdns_dns_cb(_, to_resolve, results, err)
           if err and (err ~= 'requested record is not found' and err ~= 'no records with this name') then
             rspamd_logger.errx(task, 'error looking up %s: %s', to_resolve, err)
