@@ -88,13 +88,13 @@ define(["jquery", "app/common"],
             // String to array for comparison
             const eltsArray = JSON.parse(elts);
             if (eltsArray[0] < 0) {
-                common.alertMessage("alert-modal alert-error", "Spam can not be negative");
+                common.alertMessage("alert-modal alert-danger", "Spam can not be negative");
             } else if (eltsArray[1] < 0) {
-                common.alertMessage("alert-modal alert-error", "Rewrite subject can not be negative");
+                common.alertMessage("alert-modal alert-danger", "Rewrite subject can not be negative");
             } else if (eltsArray[2] < 0) {
-                common.alertMessage("alert-modal alert-error", "Probable spam can not be negative");
+                common.alertMessage("alert-modal alert-danger", "Probable spam can not be negative");
             } else if (eltsArray[3] < 0) {
-                common.alertMessage("alert-modal alert-error", "Greylist can not be negative");
+                common.alertMessage("alert-modal alert-danger", "Greylist can not be negative");
             } else if (descending(eltsArray)) {
                 common.query("saveactions", {
                     method: "POST",
@@ -105,13 +105,13 @@ define(["jquery", "app/common"],
                     server: server
                 });
             } else {
-                common.alertMessage("alert-modal alert-error", "Incorrect order of actions thresholds");
+                common.alertMessage("alert-modal alert-danger", "Incorrect order of actions thresholds");
             }
         };
 
         ui.getMaps = function () {
             const $listmaps = $("#listMaps");
-            $listmaps.closest(".card").hide();
+            common.hide($listmaps.closest(".card"));
             common.query("maps", {
                 success: function (json) {
                     const [{data}] = json;
@@ -130,14 +130,14 @@ define(["jquery", "app/common"],
                         });
 
                         const $tr = $("<tr>").append($td).append($("<td>" + item.type + "</td>"));
-                        if (!item.loaded) $tr.addClass("table-light opacity-50");
+                        if (!item.loaded) $tr.addClass("table-active opacity-50");
 
                         const $span = $('<span class="map-link">' + item.uri + "</span>").data("item", item);
                         $span.wrap("<td>").parent().appendTo($tr);
                         $("<td>" + item.description + "</td>").appendTo($tr);
                         $tr.appendTo($tbody);
                     });
-                    $listmaps.closest(".card").show();
+                    common.show($listmaps.closest(".card"));
                 },
                 server: common.getServer()
             });
@@ -191,9 +191,9 @@ define(["jquery", "app/common"],
                     if (item.editable === false || common.read_only) {
                         $("#editor").attr(editor[mode].readonly_attr);
                         icon = "fa-eye";
-                        $("#modalSaveGroup").hide();
+                        common.hide("#modalSaveGroup");
                     } else {
-                        $("#modalSaveGroup").show();
+                        common.show("#modalSaveGroup");
                     }
                     $("#modalDialog .modal-header").find("[data-fa-i2svg]").addClass(icon);
                     $("#modalTitle").html(item.uri);
